const { RecipeRepository } = require("../repository/recipeRepoSQL");
const CustomError = require("../../../errors/CustomError");
const { readFile, unlink } = require("fs").promises;
const { resolve } = require("path");

const FILE_DIRECTORY = resolve(__dirname, "../../../data/images");

// function test() {
//   const repo = new RecipeService(new RecipeRepository());
// }

// class RecipeService {
//   constructor(repository) {
//     this.repository = repository;
//   }

class RecipeService {
  constructor() {
    this.repository = new RecipeRepository();
  }
  async getOneRecipe(recipe_id, userId) {
    try {
      const recipe = await this.repository.findRecipe(recipe_id);

      if (recipe.user_id !== userId && !recipe.isShared){
        throw new CustomError("no access", 403);
      }
             
      return recipe;
    } catch (err) {
      throw err;
    }
  }

  async getAllUserRecipes(
    userId,
    resultsPerPage,
    requestedPage,
    sortBy,
    order
  ) {
    try {
      const count = await this.repository.countDocumentsByUserId(userId);

      const totalPages = count ? Math.ceil(count / resultsPerPage) : 1;
      let page;
      if (requestedPage <= 1) {
        page = 0;
      }
      if (requestedPage >= totalPages) {
        page = totalPages - 1;
      } else {
        page = requestedPage - 1;
      }

      const recipes = await this.repository.findUserRecipesAndSort(
        userId,
        sortBy,
        order,
        resultsPerPage,
        page
      );
      return { totalPages, page, resultsPerPage, count, recipes };
    } catch (err) {
      throw err;
    }
  }

  async getAllSharedRecipes(
    userId,
    isShared,
    resultsPerPage,
    requestedPage,
    sortBy,
    order
  ) {
    try {
      const count = await this.repository.countSharedRecipes(isShared);

      const totalPages = count ? Math.ceil(count / resultsPerPage) : 1;
      let page;
      if (requestedPage <= 1) {
        page = 0;
      }
      if (requestedPage >= totalPages) {
        page = totalPages - 1;
      } else {
        page = requestedPage - 1;
      }

      const recipes = await this.repository.findSharedRecipesAndSort(
        userId,
        isShared,
        sortBy,
        order,
        resultsPerPage,
        page
      );
      return { totalPages, page, resultsPerPage, count, recipes };
    } catch (err) {
      throw err;
    }
  }

  async createRecipe(imageName, userId, data) {
    try {
      const recipeData = JSON.parse(data);
      if (!imageName) throw new CustomError("image is required", 400);
      const imageLink = `/api/v1/recipe/image=${imageName}`;
      await this.repository.saveRecipe(recipeData, imageLink, userId);

      return imageLink;
    } catch (err) {
      throw err;
    }
  }

  async editRecipe(imageName, userId, recipeId, patchData) {
    try {
      const recipePatchData = JSON.parse(patchData);

      const image = recipePatchData.coverPhoto;
      const imageLink =
        imageName && `/api/v1/recipe/image=${imageName}`;

      await this.repository.findRecipeAndUpdate(
        recipeId,
        userId,
        recipePatchData,
        imageLink
      );

      if (imageLink) {
        const oldImageRequestSplit = image.split("=");
        const oldImageFileName = oldImageRequestSplit[1];
        const oldImagePath = resolve(FILE_DIRECTORY, `./${oldImageFileName}`);
        await unlink(oldImagePath);
      }
      return imageLink ? imageLink : image;
    } catch (err) {
      throw err;
    }
  }
  async removeRecipe(recipeId, userId) {
    try {
      const image = await this.repository.findRecipeAndDelete(recipeId, userId);

      const oldImageRequestSplit = image.split("=");
      const oldImageFileName = oldImageRequestSplit[1];
      const oldImagePath = resolve(FILE_DIRECTORY, `./${oldImageFileName}`);
      await unlink(oldImagePath);

      return;
    } catch (err) {
      throw err;
    }
  }
  async getImage(image_id) {
    try {
      const imagePath = resolve(FILE_DIRECTORY, `${image_id}`);
      return await readFile(imagePath);
    } catch (err) {
      throw err;
    }
  }
}

module.exports = { RecipeService };
